import {IQueryCriteria, IQueryExecutionGroupResult, IQueryExecutionResult} from "@openharbor/data";
import {Observable, of} from "rxjs";
import {switchMap} from "rxjs/operators";
import {BaseHttpDataSourceOptionsBuilder} from "./BaseHttpDataSourceOptionsBuilder";
import {HttpClient} from "@angular/common/http";

export class HttpDataSourceOptionsBuilder<TQuery extends IQueryCriteria, TModel extends {}> extends BaseHttpDataSourceOptionsBuilder<TQuery, TModel>
{
  constructor(http: HttpClient) {
    super(http);
  }

  private _beforeRead?: (query: IQueryCriteria) => Observable<TQuery>;

  public beforeRead(beforeRead: (query: IQueryCriteria) => Observable<TQuery>) {
    this._beforeRead = beforeRead;
    return this;
  }

  public queryUrl(url: string) {
    this._query = {
      adapter: {
        handle: (query: IQueryCriteria) => {
          const finalBeforeRead = this._beforeRead || (() => of(query));
          return finalBeforeRead(query)
            .pipe(
              switchMap(finalQuery => {
                return this.http.post<IQueryExecutionResult<TModel> & IQueryExecutionGroupResult<TModel>>(url, finalQuery);
              })
            );
        }
      }
    }

    return this;
  }

  public queryHandler(queryHandler: (query: TQuery) => Observable<IQueryExecutionResult<TModel> & IQueryExecutionGroupResult<TModel>>) {
    this._query = {
      adapter: {
        handle: (query: TQuery) => {
          const finalBeforeRead = this._beforeRead || (() => of(query));
          return finalBeforeRead(query)
            .pipe(
              switchMap(finalQuery => {
                return queryHandler(finalQuery as any);
              })
            );
        }
      }
    }
    return this;
  }

  defaultCriteria(criteria: IQueryCriteria) {
    this._defaultCriteria = criteria;
    return this;
  }
}
