import {IQueryCriteria, IQueryExecutionGroupResult, IQueryExecutionResult} from '@openharbor/data';
import {Observable, of} from 'rxjs';
import {map, switchMap} from 'rxjs/operators';
import {BaseHttpDataSourceOptionsBuilder} from './BaseHttpDataSourceOptionsBuilder';
import {HttpClient} from "@angular/common/http";

export class SingleDataSourceOptionsBuilder<TQuery extends IQueryCriteria, TModel extends {}>
  extends BaseHttpDataSourceOptionsBuilder<TQuery, TModel> {
  private _beforeRead?: (query: IQueryCriteria) => Observable<TQuery>;

  constructor(http: HttpClient) {
    super(http);
  }

  public beforeRead(beforeRead: (query: IQueryCriteria) => Observable<TQuery>) {
    this._beforeRead = beforeRead;
    return this;
  }

  public queryUrlWithGet(url: string) {
    this._query = {
      adapter: {
        handle: (query: IQueryCriteria) => {
          const finalBeforeRead = this._beforeRead || (() => of({} as TQuery));
          return finalBeforeRead(query)
            .pipe(
              switchMap(finalQuery => {
                return this.http.get<TModel>(url, {
                  params: this.convertToParams(finalQuery)
                }).pipe(
                  map(result => {
                    return {
                      totalRecords: result == null ? 0 : 1,
                      data: [result]
                    } as IQueryExecutionResult<TModel> & IQueryExecutionGroupResult<TModel>;
                  })
                );
              })
            );
        }
      }
    };

    return this;
  }

  protected convertToParams(finalQuery: TQuery) {
    return Object.keys(finalQuery).reduce((prev, key) => {
      prev[key] = (finalQuery as any)[key]; // Cast finalQuery to any to bypass the error
      return prev;
    }, {} as { [param: string]: string | string[]; });
  }

  public queryUrl(url: string) {
    this._query = {
      adapter: {
        handle: (query: IQueryCriteria) => {
          const finalBeforeRead = this._beforeRead || ((_: IQueryCriteria) => of({} as TQuery));
          return finalBeforeRead(query)
            .pipe(
              switchMap(finalQuery => {
                return this.http.post<TModel>(url, finalQuery).pipe(
                  map(result => {
                    return {
                      totalRecords: result == null ? 0 : 1,
                      data: [result]
                    } as IQueryExecutionResult<TModel> & IQueryExecutionGroupResult<TModel>;
                  })
                );
              })
            );
        }
      }
    };

    return this;
  }

  public queryHandler(queryHandler: (query: IQueryCriteria) => Observable<TModel>) {
    this._query = {
      adapter: {
        handle: (query: TQuery) => {
          const finalBeforeRead = this._beforeRead || (() => of({}));
          return finalBeforeRead(query)
            .pipe(
              switchMap(finalQuery => {
                return queryHandler(finalQuery).pipe(
                  map(result => {
                    return {
                      totalRecords: result == null ? 0 : 1,
                      data: [result]
                    } as IQueryExecutionResult<TModel> & IQueryExecutionGroupResult<TModel>;
                  })
                );
              })
            );
        }
      }
    };

    return this;
  }
}
