import {
  DataSource,
  IDataSource,
  IDataSourceCommandAdapterOptions,
  IDataSourceOptions,
  IDataSourceQueryAdapterOptions,
  IDataSourceTransportOptions,
  IQueryCriteria
} from "@openharbor/data";

export abstract class InMemoryDataSourceOptionBuilderAbstraction<TQuery, TModel> {
  protected _commands: { [key: string]: IDataSourceCommandAdapterOptions<any>; } = {};
  protected _keyResolver: (model: TModel) => TModel[keyof TModel];
  protected _defaultCriteria: IQueryCriteria;
  protected _query: IDataSourceQueryAdapterOptions<TQuery, TModel>;

  createDataSource(): IDataSource<TQuery, TModel> {
    return new DataSource<TQuery, TModel>(this.createOptions());
  }

  createOptions(): IDataSourceOptions<TQuery, TModel> {
    return {
      resolveIdField: this._keyResolver,
      defaultCriteria: this._defaultCriteria,
      transport: this.createTransport()
    };
  }

  protected createTransport(): IDataSourceTransportOptions<TQuery, TModel> {
    return {
      query: this._query,
      commands: this._commands
    };
  }
}
